@@ -13,7 +13,7 @@ use warnings;
 no warnings qw(once);
 
 BEGIN {
-  $REST::Neo4p::Batch::VERSION = '0.3030';
+  $REST::Neo4p::Batch::VERSION = '0.4001';
 }
 
 our @EXPORT = qw(batch);
@@ -23,6 +23,9 @@ our $BUFSIZE = 50000;
 sub batch (&@) {
   my ($coderef,$action) = @_;
   my $agent = REST::Neo4p->agent;
+  if ($agent->is_version_4) {
+    REST::Neo4p::NotSuppException->throw("Batch mode not supported on Neo4j server v4.0+");
+  }
   my @errors;
   REST::Neo4p::CommException->throw("Not connected\n") unless REST::Neo4p->connected;
   warn 'Agent already in batch_mode on batch() call' if ($agent->batch_mode);
@@ -143,6 +146,9 @@ REST::Neo4p::Batch - Mixin for batch processing
 REST::Neo4p::Batch adds some syntactic sugar allowing ordinary
 REST::Neo4p code to be processed through the Neo4j REST batch API.
 
+Batch mode is not supported in Neo4j version 4.0+. The methods in this
+module will barf.
+
 =head1 batch {} ($action)
 
 To execute server calls generated by REST::Neo4p code, 
@@ -244,7 +250,7 @@ L<REST::Neo4p>, L<REST::Neo4p::Agent>
 
 =head1 LICENSE
 
-Copyright (c) 2012-2017 Mark A. Jensen. This program is free software; you
+Copyright (c) 2012-2021 Mark A. Jensen. This program is free software; you
 can redistribute it and/or modify it under the same terms as Perl
 itself.
 
